@@ -3,7 +3,7 @@ package Term::Choose::Win32;
 use 5.10.0;
 use strict;
 
-our $VERSION = '0.010';
+our $VERSION = '0.011';
 use Exporter 'import';
 our @EXPORT_OK = qw(choose);
 
@@ -367,10 +367,10 @@ sub choose {
                 if ( defined $arg->{backup_row} ) {
                     $arg->{backup_row} = undef;
                 }
-                #if ( defined $arg->{backup_col} ) {
-                #    $arg->{cursor}[COL] = $arg->{backup_col};
-                #    $arg->{backup_col} = undef;
-                #}
+                if ( defined $arg->{backup_col} ) {
+                    $arg->{cursor}[COL] = $arg->{backup_col};
+                    $arg->{backup_col}  = undef;
+                }
                 if ( $arg->{cursor}[ROW] >= $arg->{p_begin} ) {
                     _wr_cell( $arg, $arg->{cursor}[ROW] + 1, $arg->{cursor}[COL] );
                     _wr_cell( $arg, $arg->{cursor}[ROW],     $arg->{cursor}[COL] );
@@ -419,9 +419,9 @@ sub choose {
                 _beep( $arg );
             }
             else {
-                #if ( defined $arg->{backup_col} ) {
-                #    $arg->{backup_col} = undef;
-                #}
+                if ( defined $arg->{backup_col} ) {
+                    $arg->{backup_col} = undef;
+                }
                 if ( $arg->{cursor}[COL] > 0 ) {
                     $arg->{cursor}[COL]--;
                     _wr_cell( $arg, $arg->{cursor}[ROW], $arg->{cursor}[COL] + 1 );
@@ -464,9 +464,9 @@ sub choose {
             }
             else {
                 $arg->{cursor}[COL]--;
-                #if ( defined $arg->{backup_col} ) {
-                #    $arg->{backup_col} = undef;
-                #}
+                if ( defined $arg->{backup_col} ) {
+                    $arg->{backup_col} = undef;
+                }
                 _wr_cell( $arg, $arg->{cursor}[ROW], $arg->{cursor}[COL] + 1 );
                 _wr_cell( $arg, $arg->{cursor}[ROW], $arg->{cursor}[COL] );
             }
@@ -482,10 +482,10 @@ sub choose {
                     $arg->{cursor}[ROW] = $arg->{backup_row};
                     $arg->{backup_row}  = undef;
                 }
-                #if ( defined $arg->{backup_col} ) {
-                #    $arg->{cursor}[COL] = $arg->{backup_col};
-                #    $arg->{backup_col} = undef;
-                #}
+                if ( defined $arg->{backup_col} ) {
+                    $arg->{cursor}[COL] = $arg->{backup_col};
+                    $arg->{backup_col}  = undef;
+                }
                 $arg->{p_begin} = $arg->{row_on_top};
                 $arg->{p_end}   = $arg->{p_begin} + $arg->{avail_height} - 1;
                 _wr_screen( $arg );
@@ -498,22 +498,22 @@ sub choose {
             else {
                 $arg->{row_on_top} = $arg->{avail_height} * ( int( $arg->{cursor}[ROW] / $arg->{avail_height} ) + 1 );
                 $arg->{cursor}[ROW] += $arg->{avail_height};
-                if ( $arg->{cursor}[ROW] > $#{$arg->{rc2idx}} ) {
-                    $arg->{backup_row}  = $arg->{cursor}[ROW] - $arg->{avail_height};
-                    if ( $#{$arg->{rc2idx}} == $arg->{row_on_top} || $arg->{cursor}[COL] <= $arg->{rest} - 1 ) {
-                        $arg->{cursor}[ROW] = $#{$arg->{rc2idx}};
+                if ( $arg->{cursor}[ROW] >= $#{$arg->{rc2idx}} ) {
+                    if ( $#{$arg->{rc2idx}} == $arg->{row_on_top} || ! $arg->{rest} || $arg->{cursor}[COL] <= $arg->{rest} - 1 ) {
+                        if ( $arg->{cursor}[ROW] != $#{$arg->{rc2idx}} ) {
+                            $arg->{backup_row}  = $arg->{cursor}[ROW] - $arg->{avail_height};
+                            $arg->{cursor}[ROW] = $#{$arg->{rc2idx}};
+                        }
+                        if ( $arg->{rest} && $arg->{cursor}[COL] > $arg->{rest} - 1 ) {
+                            $arg->{backup_col}  = $arg->{cursor}[COL];
+                            $arg->{cursor}[COL] = $#{$arg->{rc2idx}[$arg->{cursor}[ROW]]};
+                        }
                     }
                     else {
+                        $arg->{backup_row}  = $arg->{cursor}[ROW] - $arg->{avail_height};
                         $arg->{cursor}[ROW] = $#{$arg->{rc2idx}} - 1;
                     }
                 }
-                if (    $arg->{rest}
-                     && $arg->{cursor}[ROW] == $#{$arg->{rc2idx}}
-                     && $arg->{cursor}[COL] > $arg->{rest} - 1
-                ) {
-                #    $arg->{backup_col} = $arg->{cursor}[COL];
-                    $arg->{cursor}[COL] = $#{$arg->{rc2idx}[$arg->{cursor}[ROW]]};
-                }
                 $arg->{p_begin} = $arg->{row_on_top};
                 $arg->{p_end}   = $arg->{p_begin} + $arg->{avail_height} - 1;
                 $arg->{p_end}   = $#{$arg->{rc2idx}} if $arg->{p_end} > $#{$arg->{rc2idx}};
@@ -834,7 +834,7 @@ Term::Choose::Win32 - Choose items from a list.
 
 =head1 VERSION
 
-Version 0.010
+Version 0.011
 
 =cut
 
